@@ -3,11 +3,11 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Generate walking basslines
 
-our $VERSION = '0.0404';
+our $VERSION = '0.0500';
 
 use Data::Dumper::Compact qw(ddc);
 use Carp qw(croak);
-use List::MoreUtils qw(first_index);
+use List::SomeUtils qw(first_index);
 use List::Util qw(any min uniq);
 use Music::Chord::Note;
 use Music::Note;
@@ -66,9 +66,27 @@ has octave => (
 has scale => (
     is      => 'ro',
     isa     => sub { croak 'not a code reference' unless ref $_[0] eq 'CODE' },
-    default => sub { sub { $_[0] =~ /^[A-G][#b]?m/ ? 'minor' : 'major' } },
+    builder => 1,
 );
 
+sub _build_scale {
+    my ($self) = @_;
+    if ($self->modal) {
+        return sub {
+            my ($chord) = @_;
+            my ($chord_note) = _parse_chord($chord);
+            my @modes = qw( ionian dorian phrygian lydian mixolydian aeolian locrian );
+            my @key_notes = get_scale_notes($self->keycenter, $modes[0]);
+            my $position = first_index { $_ eq $chord_note } @key_notes;
+            my $scale = $position >= 0 ? $modes[$position] : $modes[0];
+            return $scale;
+        };
+    }
+    else {
+        return sub { $_[0] =~ /^[A-G][#b]?m/ ? 'minor' : 'major' };
+    }
+}
+
 
 has tonic => (
     is      => 'ro',
@@ -98,36 +116,14 @@ sub generate {
     print "CHORD: $chord\n" if $self->verbose;
     print "NEXT: $next_chord\n" if $self->verbose && $next_chord;
 
-    # Parse the chord
-    my $chord_note;
-    my $flavor;
-    if ($chord =~ /^([A-G][#b]?)(.*)$/) {
-        $chord_note = $1;
-        $flavor = $2;
-    }
+    my ($chord_note, $flavor) = _parse_chord($chord);
 
-    # Parse the next chord
     my $next_chord_note;
-    if ($next_chord && $next_chord =~ /^([A-G][#b]?).*$/) {
-        $next_chord_note = $1;
-    }
+    ($next_chord_note) = _parse_chord($next_chord)
+        if $next_chord;
 
-    my ($scale, $next_scale);
-    if ($self->modal) {
-        print "MODAL\n" if $self->verbose;
-        my @modes = qw( ionian dorian phrygian lydian mixolydian aeolian locrian );
-        my @key_notes = get_scale_notes($self->keycenter, $modes[0]);
-        my $position = first_index { $_ eq $chord_note } @key_notes;
-        $scale = $position >= 0 ? $modes[$position] : $modes[0];
-        if ($next_chord_note) {
-            $position = first_index { $_ eq $next_chord_note } @key_notes;
-            $next_scale = $position >= 0 ? $modes[$position] : $modes[0];
-        }
-    }
-    else {
-        $scale = $self->scale->($chord);
-        $next_scale = defined $next_chord ? $self->scale->($next_chord) : '';
-    }
+    my $scale = $self->scale->($chord);
+    my $next_scale = defined $next_chord ? $self->scale->($next_chord) : '';
 
     my $cn = Music::Chord::Note->new;
 
@@ -191,7 +187,6 @@ sub generate {
 
     # Make sure there are no duplicate pitches
     @fixed = uniq @fixed;
-
     $self->_verbose_notes('NOTES', @fixed) if $self->verbose;
 
     my $voice = Music::VoiceGen->new(
@@ -205,13 +200,14 @@ sub generate {
     # Get a passage of quasi-random pitches
     my @chosen = map { $voice->rand } 1 .. $num;
 
+    # Choose the right note given the scale if the tonic is set
     if ($self->tonic) {
-        if ($scale eq 'major' || $scale eq 'minor') {
-            $chosen[0] = _closest($chosen[1], [ @fixed[0,2,4] ])
-        }
-        elsif ($scale eq 'pentatonic' || $scale eq 'pminor') {
+        if ($scale eq 'pentatonic' || $scale eq 'pminor') {
             $chosen[0] = _closest($chosen[1], [ @fixed[0,1,2] ])
         }
+        elsif (@fixed == 7) { # standard, 7-note Western scale
+            $chosen[0] = _closest($chosen[1], [ @fixed[0,2,4] ])
+        }
     }
 
     # Intersect with the next-chord pitches
@@ -234,6 +230,17 @@ sub generate {
     return \@chosen;
 }
 
+sub _parse_chord {
+  my ($chord) = @_;
+    my $chord_note;
+    my $flavor;
+    if ($chord =~ /^([A-G][#b]?)(.*)$/) {
+        $chord_note = $1;
+        $flavor = $2;
+    }
+    return $chord_note, $flavor;
+}
+
 # Show a phrase of midinums as ISO notes
 sub _verbose_notes {
     my ($self, $title, @notes) = @_;
@@ -274,7 +281,7 @@ MIDI::Bassline::Walk - Generate walking basslines
 
 =head1 VERSION
 
-version 0.0404
+version 0.0500
 
 =head1 SYNOPSIS
 
@@ -370,6 +377,10 @@ Default if not B<modal>:
 
   sub { $_[0] =~ /^[A-G][#b]?m/ ? 'minor' : 'major' }
 
+Otherwise, select the appropriate mode (ionian, dorian, phrygian,
+lydian, mixolydian, aeolian, locrian) given the named B<chord> and the
+modal B<keycenter>.
+
 Alternatives:
 
   sub { 'chromatic' }
